/* USER CODE BEGIN Header */
/**
  ******************************************************************************
  * File Name          : freertos.c
  * Description        : Code for freertos applications
  ******************************************************************************
  * @attention
  *
  * Copyright (c) 2025 STMicroelectronics.
  * All rights reserved.
  *
  * This software is licensed under terms that can be found in the LICENSE file
  * in the root directory of this software component.
  * If no LICENSE file comes with this software, it is provided AS-IS.
  *
  ******************************************************************************
  */
/* USER CODE END Header */

/* Includes ------------------------------------------------------------------*/
#include "FreeRTOS.h"
#include "task.h"
#include "main.h"
#include "cmsis_os.h"

/* Private includes ----------------------------------------------------------*/
/* USER CODE BEGIN Includes */
#include <stdio.h>

#include "esp8266.h"
#include "OneNet.h"
#include "oled.h"
#include "stm32f1xx_hal.h"
#include <string.h>
#include "retarget.h"
#include "tim.h"
#include "timers.h"
/* USER CODE END Includes */

/* Private typedef -----------------------------------------------------------*/
/* USER CODE BEGIN PTD */

/* USER CODE END PTD */

/* Private define ------------------------------------------------------------*/
/* USER CODE BEGIN PD */
#define K1 HAL_GPIO_ReadPin(KEY1_GPIO_Port, KEY1_Pin)
#define K2 HAL_GPIO_ReadPin(KEY2_GPIO_Port, KEY2_Pin)
#define K3 HAL_GPIO_ReadPin(KEY3_GPIO_Port, KEY3_Pin)
#define K4 HAL_GPIO_ReadPin(KEY4_GPIO_Port, KEY4_Pin)
/* USER CODE END PD */

/* Private macro -------------------------------------------------------------*/
/* USER CODE BEGIN PM */

/* USER CODE END PM */

/* Private variables ---------------------------------------------------------*/
/* USER CODE BEGIN Variables */
char ONENET_CONNECTED = 0;

// Flags
char KEY_PRESSED[4] = {0, 0, 0, 0};
/* USER CODE END Variables */
/* Definitions for defaultTask */
osThreadId_t defaultTaskHandle;
const osThreadAttr_t defaultTask_attributes = {
  .name = "defaultTask",
  .stack_size = 256 * 4,
  .priority = (osPriority_t) osPriorityNormal,
};
/* Definitions for keyScanTask */
osThreadId_t keyScanTaskHandle;
const osThreadAttr_t keyScanTask_attributes = {
  .name = "keyScanTask",
  .stack_size = 128 * 4,
  .priority = (osPriority_t) osPriorityNormal,
};
/* Definitions for cmdTask */
osThreadId_t cmdTaskHandle;
const osThreadAttr_t cmdTask_attributes = {
  .name = "cmdTask",
  .stack_size = 256 * 4,
  .priority = (osPriority_t) osPriorityNormal,
};

/* Private function prototypes -----------------------------------------------*/
/* USER CODE BEGIN FunctionPrototypes */
void KeyScan(void);
void Display_IM1281B(void);
/* USER CODE END FunctionPrototypes */

void StartDefaultTask(void *argument);
void startKeyScanTask(void *argument);
void startCmdTask(void *argument);

void MX_FREERTOS_Init(void); /* (MISRA C 2004 rule 8.1) */

/**
  * @brief  FreeRTOS initialization
  * @param  None
  * @retval None
  */
void MX_FREERTOS_Init(void) {
  /* USER CODE BEGIN Init */
  OLED_ShowString(0,0,"initializing...",12);

  int const isSmartConfigMode = !HAL_GPIO_ReadPin(KEY1_GPIO_Port, KEY1_Pin);

  if (isSmartConfigMode) OLED_ShowString(0,0,"smartConfigMode",12);
  ESP8266_Init(isSmartConfigMode);

  OLED_ShowString(0, 0, "connectOneNet", 12);

  while (OneNet_DevLink()) {}
  OneNET_Subscribe();
  ONENET_CONNECTED = 1;

  OLED_ShowString(0,0,"OneNet online            ",12);
  HAL_GPIO_WritePin(LED0_GPIO_Port, LED0_Pin, GPIO_PIN_RESET);

  HAL_TIM_Base_Start_IT(&htim1);
  /* USER CODE END Init */

  /* USER CODE BEGIN RTOS_MUTEX */
  /* add mutexes, ... */
  /* USER CODE END RTOS_MUTEX */

  /* USER CODE BEGIN RTOS_SEMAPHORES */
  /* add semaphores, ... */
  /* USER CODE END RTOS_SEMAPHORES */

  /* USER CODE BEGIN RTOS_TIMERS */
  /* start timers, add new ones, ... */
  /* USER CODE END RTOS_TIMERS */

  /* USER CODE BEGIN RTOS_QUEUES */
  /* add queues, ... */
  /* USER CODE END RTOS_QUEUES */

  /* Create the thread(s) */
  /* creation of defaultTask */
  defaultTaskHandle = osThreadNew(StartDefaultTask, NULL, &defaultTask_attributes);

  /* creation of keyScanTask */
  keyScanTaskHandle = osThreadNew(startKeyScanTask, NULL, &keyScanTask_attributes);

  /* creation of cmdTask */
  cmdTaskHandle = osThreadNew(startCmdTask, NULL, &cmdTask_attributes);

  /* USER CODE BEGIN RTOS_THREADS */
  /* add threads, ... */
  /* USER CODE END RTOS_THREADS */

  /* USER CODE BEGIN RTOS_EVENTS */
  /* add events, ... */
  /* USER CODE END RTOS_EVENTS */

}

/* USER CODE BEGIN Header_StartDefaultTask */
/**
  * @brief  Function implementing the defaultTask thread.
  * @param  argument: Not used
  * @retval None
  */
/* USER CODE END Header_StartDefaultTask */
void StartDefaultTask(void *argument)
{
  /* USER CODE BEGIN StartDefaultTask */
  /* Infinite loop */
  for(;;)
  {

    // osDelay(500);
    //
    // OneNet_SendData();
    // osDelay(500);
    // ESP8266_Clear();

    if (KEY_PRESSED[0] || KEY_PRESSED[1])
    {
      KEY_PRESSED[0] = 0;
      KEY_PRESSED[1] = 0;
      HAL_GPIO_TogglePin(RELAY1_GPIO_Port, RELAY1_Pin);
    }


    if (flag_1s)
    {
      flag_1s = 0;
      IM_ReadFlag = 1;
    }

    if (flag_2s)
    {
      flag_2s = 0;
      HAL_GPIO_TogglePin(LED0_GPIO_Port, LED0_Pin);
    }

    if (flag_5s)
    {
      flag_5s = 0;
      OneNet_SendData();
      ESP8266_Clear();
    }

    if (IM_RecvDone)
    {
      IM_Analyze();
      Display_IM1281B();
    }

    IM_Read();
  }
  /* USER CODE END StartDefaultTask */
}

/* USER CODE BEGIN Header_startKeyScanTask */
/**
* @brief Function implementing the keyScanTask thread.
* @param argument: Not used
* @retval None
*/
/* USER CODE END Header_startKeyScanTask */
void startKeyScanTask(void *argument)
{
  /* USER CODE BEGIN startKeyScanTask */
  /* Infinite loop */
  for(;;)
  {
    KeyScan();
  }
  /* USER CODE END startKeyScanTask */
}

/* USER CODE BEGIN Header_startCmdTask */
/**
* @brief Function implementing the cmdTask thread.
* @param argument: Not used
* @retval None
*/
/* USER CODE END Header_startCmdTask */
void startCmdTask(void *argument)
{
  /* USER CODE BEGIN startCmdTask */
  /* Infinite loop */
  for(;;)
  {
    // if (ONENET_CONNECTED)
    // {
    //   uint8_t* dataPtr = ESP8266_GetIPD_soft(0);
    //   if (dataPtr != NULL)
    //   {
    //     OneNet_RevPro(dataPtr);
    //   }
    // }
  }
  /* USER CODE END startCmdTask */
}

/* Private application code --------------------------------------------------*/
/* USER CODE BEGIN Application */
void KeyScan(void)
{
  if (K1 == GPIO_PIN_RESET)
  {
    osDelay(5);
    if (K1 == GPIO_PIN_RESET)
    {
      KEY_PRESSED[0] = 1;
    }
    while (K1 == GPIO_PIN_RESET);
  }
  if (K2 == GPIO_PIN_RESET)
  {
    osDelay(5);
    if (K2 == GPIO_PIN_RESET)
    {
      KEY_PRESSED[1] = 1;
    }
    while (K2 == GPIO_PIN_RESET);
  }
  if (K3 == GPIO_PIN_RESET)
  {
    osDelay(5);
    if (K3 == GPIO_PIN_RESET)
    {
      KEY_PRESSED[2] = 1;
    }
    while (K3 == GPIO_PIN_RESET);
  }
  if (K4 == GPIO_PIN_RESET)
  {
    osDelay(5);
    if (K4 == GPIO_PIN_RESET)
    {
      KEY_PRESSED[3] = 1;
    }
    while (K4 == GPIO_PIN_RESET);
  }
}

void Display_IM1281B(void)
{
  OLED_ShowString(0, 0, "Volt:", 12);
  OLED_ShowString(0, 2, "Curr:", 12);
  OLED_ShowString(0, 4, "Pwr: ", 12);
  OLED_ShowString(0, 6, "Eng: ", 12);

  char buffer[16];

  snprintf(buffer, sizeof(buffer), "%.2fV", IM_Volt * 0.0001);
  OLED_ShowString(40, 0, buffer, 12);

  snprintf(buffer, sizeof(buffer), "%.2fA", IM_Curr * 0.0001);
  OLED_ShowString(40, 2, buffer, 12);

  snprintf(buffer, sizeof(buffer), "%.2fW", IM_Power * 0.0001);
  OLED_ShowString(40, 4, buffer, 12);

  snprintf(buffer, sizeof(buffer), "%.3fKWh", IM_Energy * 0.0001);
  OLED_ShowString(40, 6, buffer, 12);
}
/* USER CODE END Application */

